@@ -5,18 +5,12 @@
  *
  * Author: Guido Günther <agx@sigxcpu.org>
  */
-
-/* WIFI Manager */
-
 #define G_LOG_DOMAIN "phosh-wifimanager"
 
 #include "config.h"
 
-#include "contrib/shell-network-agent.h"
-#include "network-auth-prompt.h"
 #include "wifimanager.h"
 #include "shell.h"
-#include "phosh-wayland.h"
 #include "util.h"
 
 #include <NetworkManager.h>
@@ -26,10 +20,7 @@
  * @short_description: Tracks the Wifi status and handle wifi credentials entry
  * @Title: PhoshWifiManager
  *
- * Wi-Fi credentials are handled with #ShellNetworkAgent which implements
- * #NMSecretAgentOld.  When a credential for some wi-fi network is requested,
- * A new #PhoshNetworkAuthPrompt is created, which asks the user various
- * credentials required depending on the Access point security method.
+ * Manages wifi information and state
  */
 
 enum {
@@ -64,8 +55,6 @@ struct _PhoshWifiManager
   NMActiveConnection *active;
   /* The wifi device used in the active connection */
   NMDeviceWifi       *dev;
-  ShellNetworkAgent  *network_agent;
-  PhoshNetworkAuthPrompt *network_prompt;
 };
 G_DEFINE_TYPE (PhoshWifiManager, phosh_wifi_manager, G_TYPE_OBJECT);
 
@@ -184,16 +173,6 @@ update_state (PhoshWifiManager *self)
 }
 
 
-static void
-phosh_wifi_manager_set_property (GObject *object,
-                                 guint property_id,
-                                 const GValue *value,
-                                 GParamSpec *pspec)
-{
-  G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
-}
-
-
 static void
 phosh_wifi_manager_get_property (GObject *object,
                                  guint property_id,
@@ -463,125 +442,6 @@ on_nmclient_devices_changed (PhoshWifiManager *self, GParamSpec *pspec, NMClient
 }
 
 
-static void
-network_prompt_done_cb (PhoshWifiManager *self)
-{
-  g_return_if_fail (PHOSH_IS_WIFI_MANAGER (self));
-
-  if (self->network_prompt)
-    gtk_widget_hide (GTK_WIDGET (self->network_prompt));
-
-  g_clear_pointer ((GtkWidget **)&self->network_prompt, gtk_widget_destroy);
-}
-
-static void
-network_agent_setup_prompt (PhoshWifiManager *self)
-{
-  GtkWidget *network_prompt;
-
-  g_return_if_fail (PHOSH_IS_WIFI_MANAGER (self));
-
-  if (self->network_prompt)
-    return;
-
-  network_prompt = phosh_network_auth_prompt_new (self->network_agent,
-                                                  self->nmclient);
-  self->network_prompt = PHOSH_NETWORK_AUTH_PROMPT (network_prompt);
-
-  g_signal_connect_object (self->network_prompt, "done",
-                           G_CALLBACK (network_prompt_done_cb),
-                           self, G_CONNECT_SWAPPED);
-
-  /* Show widget when not locked and keep that in sync */
-  g_object_bind_property (phosh_shell_get_default (), "locked",
-                          self->network_prompt, "visible",
-                          G_BINDING_SYNC_CREATE | G_BINDING_INVERT_BOOLEAN);
-}
-
-
-static void
-secret_request_new_cb (PhoshWifiManager              *self,
-                       char                          *request_id,
-                       NMConnection                  *connection,
-                       char                          *setting_name,
-                       char                         **hints,
-                       NMSecretAgentGetSecretsFlags   flags,
-                       ShellNetworkAgent             *agent)
-{
-  g_return_if_fail (PHOSH_IS_WIFI_MANAGER (self));
-
-  if (!nm_connection_is_type(connection, NM_SETTING_WIRELESS_SETTING_NAME)) {
-    g_warning ("Only Wifi Networks currently supported");
-    shell_network_agent_respond (self->network_agent, request_id, SHELL_NETWORK_AGENT_USER_CANCELED);
-    return;
-  }
-
-  g_return_if_fail (!self->network_prompt);
-
-  network_agent_setup_prompt (self);
-  phosh_network_auth_prompt_set_request (self->network_prompt,
-                                         request_id, connection, setting_name,
-                                         hints, flags);
-
-}
-
-
-static void
-secret_request_cancelled_cb (PhoshWifiManager  *self,
-                             char              *request_id,
-                             ShellNetworkAgent *agent)
-{
-  g_return_if_fail (PHOSH_IS_WIFI_MANAGER (self));
-  g_return_if_fail (SHELL_IS_NETWORK_AGENT (agent));
-
-  network_prompt_done_cb (self);
-}
-
-
-static void
-secret_agent_register_cb (GObject      *object,
-                          GAsyncResult *result,
-                          gpointer      user_data)
-{
-  PhoshWifiManager *self = user_data;
-  NMSecretAgentOld *agent = NM_SECRET_AGENT_OLD (object);
-  g_autoptr(GError) error = NULL;
-
-  if (!nm_secret_agent_old_register_finish (agent, result, &error)) {
-    g_message ("Error registering network agent: %s", error->message);
-    return;
-  }
-
-  g_return_if_fail (PHOSH_IS_WIFI_MANAGER (self));
-
-  g_signal_connect_object (self->network_agent, "new-request",
-                           G_CALLBACK (secret_request_new_cb),
-                           self, G_CONNECT_SWAPPED);
-  g_signal_connect_object (self->network_agent, "cancel-request",
-                           G_CALLBACK (secret_request_cancelled_cb),
-                           self, G_CONNECT_SWAPPED);
-}
-
-static void
-setup_network_agent (PhoshWifiManager *self)
-{
-  g_autoptr(GError) error = NULL;
-
-  g_return_if_fail (PHOSH_IS_WIFI_MANAGER (self));
-
-  self->network_agent = g_initable_new (SHELL_TYPE_NETWORK_AGENT, NULL, &error,
-                                        "identifier", "sm.puri.phosh.NetworkAgent",
-                                        "auto-register", FALSE, NULL);
-
-  if (error) {
-    g_warning ("Error: %s", error->message);
-    return;
-  }
-
-  nm_secret_agent_old_register_async (NM_SECRET_AGENT_OLD (self->network_agent), NULL,
-                                      secret_agent_register_cb, self);
-}
-
 static void
 on_nm_client_ready (GObject *obj, GAsyncResult *res, gpointer data)
 {
@@ -598,7 +458,6 @@ on_nm_client_ready (GObject *obj, GAsyncResult *res, gpointer data)
   self = PHOSH_WIFI_MANAGER (data);
   self->nmclient = client;
 
-  setup_network_agent (self);
   g_signal_connect_swapped (self->nmclient, "notify::wireless-enabled",
                             G_CALLBACK (on_nmclient_wireless_enabled_changed), self);
   g_signal_connect_swapped (self->nmclient, "notify::active-connections",
@@ -640,7 +499,6 @@ phosh_wifi_manager_dispose (GObject *object)
     g_clear_object (&self->ap);
   }
 
-  g_clear_object (&self->network_agent);
   if (self->nmclient) {
     g_signal_handlers_disconnect_by_data (self->nmclient, self);
     g_clear_object (&self->nmclient);
@@ -665,7 +523,6 @@ phosh_wifi_manager_class_init (PhoshWifiManagerClass *klass)
   object_class->constructed = phosh_wifi_manager_constructed;
   object_class->dispose = phosh_wifi_manager_dispose;
 
-  object_class->set_property = phosh_wifi_manager_set_property;
   object_class->get_property = phosh_wifi_manager_get_property;
 
   props[PHOSH_WIFI_MANAGER_PROP_ICON_NAME] =
